@@ -3,7 +3,18 @@ import re
 import uuid
 from dataclasses import is_dataclass
 from threading import local
-from typing import TYPE_CHECKING, Any, Callable, Dict, Mapping, Optional, Type, TypeVar
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    Dict,
+    Iterable,
+    Mapping,
+    Optional,
+    Tuple,
+    Type,
+    TypeVar,
+)
 
 import attr
 from attr import NOTHING, frozen, resolve_types
@@ -12,6 +23,7 @@ from cattrs.errors import (
     ClassValidationError,
     ForbiddenExtraKeysError,
     IterableValidationError,
+    StructureHandlerNotFoundError,
 )
 
 from ._compat import (
@@ -25,7 +37,7 @@ from ._compat import (
 from ._generics import deep_copy_with
 
 if TYPE_CHECKING:  # pragma: no cover
-    from cattr.converters import Converter
+    from cattr.converters import BaseConverter
 
 
 @frozen
@@ -35,7 +47,11 @@ class AttributeOverride:
     omit: bool = False  # Omit the field completely.
 
 
-def override(omit_if_default=None, rename=None, omit: bool = False):
+def override(
+    omit_if_default: Optional[bool] = None,
+    rename: Optional[str] = None,
+    omit: bool = False,
+):
     return AttributeOverride(omit_if_default=omit_if_default, rename=rename, omit=omit)
 
 
@@ -45,12 +61,12 @@ T = TypeVar("T")
 
 
 def make_dict_unstructure_fn(
-    cl,
-    converter,
+    cl: Type[T],
+    converter: "BaseConverter",
     _cattrs_omit_if_default: bool = False,
     _cattrs_use_linecache: bool = True,
-    **kwargs,
-) -> Callable[[Any], Dict]:
+    **kwargs: AttributeOverride,
+) -> Callable[[T], Dict[str, Any]]:
     """
     Generate a specialized dict unstructuring function for an attrs class or a
     dataclass.
@@ -213,15 +229,18 @@ def _generate_mapping(cl: Type, old_mapping: Dict[str, type]) -> Dict[str, type]
     return mapping
 
 
+DictStructureFn = Callable[[Mapping[str, Any], Any], T]
+
+
 def make_dict_structure_fn(
     cl: Type[T],
-    converter: "Converter",
+    converter: "BaseConverter",
     _cattrs_forbid_extra_keys: bool = False,
     _cattrs_use_linecache: bool = True,
     _cattrs_prefer_attrib_converters: bool = False,
     _cattrs_detailed_validation: bool = True,
     **kwargs: AttributeOverride,
-) -> Callable[[Mapping[str, Any], Any], T]:
+) -> DictStructureFn[T]:
     """Generate a specialized dict structuring function for an attrs class."""
 
     mapping = {}
@@ -244,7 +263,13 @@ def make_dict_structure_fn(
     # We have generic parameters and need to generate a unique name for the function
     for p in getattr(cl, "__parameters__", ()):
         # This is nasty, I am not sure how best to handle `typing.List[str]` or `TClass[int, int]` as a parameter type here
-        name_base = mapping[p.__name__]
+        try:
+            name_base = mapping[p.__name__]
+        except KeyError:
+            raise StructureHandlerNotFoundError(
+                f"Missing type for generic argument {p.__name__}, specify it when structuring.",
+                p,
+            ) from None
         name = getattr(name_base, "__name__", None) or str(name_base)
         name = re.sub(r"[\[\.\] ,]", "_", name)
         fn_name += f"_{name}"
@@ -329,7 +354,7 @@ def make_dict_structure_fn(
             lines.append(f"{i}except Exception as e:")
             i = f"{i}  "
             lines.append(
-                f"{i}e.__note__ = 'Structuring class {cl.__qualname__} @ attribute {an}'"
+                f"{i}e.__notes__ = getattr(e, '__notes__', []) + [\"Structuring class {cl.__qualname__} @ attribute {an}\"]"
             )
             lines.append(f"{i}errors.append(e)")
 
@@ -341,7 +366,7 @@ def make_dict_structure_fn(
             ]
 
         post_lines.append(
-            f"  if errors: raise __c_cve('While structuring {cl.__name__}', errors, __cl)"
+            f"  if errors: raise __c_cve('While structuring ' + {cl.__name__!r}, errors, __cl)"
         )
         instantiation_lines = (
             ["  try:"]
@@ -349,7 +374,7 @@ def make_dict_structure_fn(
             + [f"      {line}" for line in invocation_lines]
             + ["    )"]
             + [
-                f"  except Exception as exc: raise __c_cve('While structuring {cl.__name__}', [exc], __cl)"
+                f"  except Exception as exc: raise __c_cve('While structuring ' + {cl.__name__!r}, [exc], __cl)"
             ]
         )
     else:
@@ -395,15 +420,18 @@ def make_dict_structure_fn(
                 internal_arg_parts[struct_handler_name] = handler
                 if handler == converter._structure_call:
                     internal_arg_parts[struct_handler_name] = t
-                    invocation_lines.append(f"{struct_handler_name}(o['{kn}']),")
+                    invocation_line = f"{struct_handler_name}(o['{kn}']),"
                 else:
                     type_name = f"__c_type_{an}"
                     internal_arg_parts[type_name] = t
-                    invocation_lines.append(
-                        f"{struct_handler_name}(o['{kn}'], {type_name}),"
-                    )
+                    invocation_line = f"{struct_handler_name}(o['{kn}'], {type_name}),"
             else:
-                invocation_lines.append(f"o['{kn}'],")
+                invocation_line = f"o['{kn}'],"
+
+            if a.kw_only:
+                ian = an if (is_dc or an[0] != "_") else an[1:]
+                invocation_line = f"{ian}={invocation_line}"
+            invocation_lines.append(invocation_line)
 
         # The second loop is for optional args.
         if non_required:
@@ -490,7 +518,12 @@ def make_dict_structure_fn(
     return globs[fn_name]
 
 
-def make_iterable_unstructure_fn(cl: Any, converter, unstructure_to=None):
+IterableUnstructureFn = Callable[[Iterable[Any]], Any]
+
+
+def make_iterable_unstructure_fn(
+    cl: Any, converter: "BaseConverter", unstructure_to: Any = None
+) -> IterableUnstructureFn:
     """Generate a specialized unstructure function for an iterable."""
     handler = converter.unstructure
 
@@ -521,7 +554,12 @@ def make_iterable_unstructure_fn(cl: Any, converter, unstructure_to=None):
     return fn
 
 
-def make_hetero_tuple_unstructure_fn(cl: Any, converter, unstructure_to=None):
+HeteroTupleUnstructureFn = Callable[[Tuple[Any, ...]], Any]
+
+
+def make_hetero_tuple_unstructure_fn(
+    cl: Any, converter: "BaseConverter", unstructure_to: Any = None
+) -> HeteroTupleUnstructureFn:
     """Generate a specialized unstructure function for a heterogenous tuple."""
     fn_name = "unstructure_tuple"
 
@@ -562,9 +600,15 @@ def make_hetero_tuple_unstructure_fn(cl: Any, converter, unstructure_to=None):
     return fn
 
 
+MappingUnstructureFn = Callable[[Mapping[Any, Any]], Any]
+
+
 def make_mapping_unstructure_fn(
-    cl: Any, converter, unstructure_to=None, key_handler=None
-):
+    cl: Any,
+    converter: "BaseConverter",
+    unstructure_to: Any = None,
+    key_handler: Optional[Callable[[Any, Optional[Any]], Any]] = None,
+) -> MappingUnstructureFn:
     """Generate a specialized unstructure function for a mapping."""
     kh = key_handler or converter.unstructure
     val_handler = converter.unstructure
@@ -613,18 +657,21 @@ def make_mapping_unstructure_fn(
     return fn
 
 
+MappingStructureFn = Callable[[Mapping[Any, Any], Any], T]
+
+
 def make_mapping_structure_fn(
     cl: Type[T],
-    converter: "Converter",
-    structure_to=dict,
+    converter: "BaseConverter",
+    structure_to: Type = dict,
     key_type=NOTHING,
     val_type=NOTHING,
     detailed_validation: bool = True,
-) -> Callable[[Mapping, Any], T]:
+) -> MappingStructureFn[T]:
     """Generate a specialized unstructure function for a mapping."""
     fn_name = "structure_mapping"
 
-    globs = {"__cattr_mapping_cl": structure_to}
+    globs: Dict[str, Type] = {"__cattr_mapping_cl": structure_to}
 
     lines = []
     lines.append(f"def {fn_name}(mapping, _):")
@@ -689,7 +736,7 @@ def make_mapping_structure_fn(
             lines.append(f"      value = {v_s}")
             lines.append("    except Exception as e:")
             lines.append(
-                "      e.__note__ = 'Structuring mapping value @ key ' + repr(k)"
+                "      e.__notes__ = getattr(e, '__notes__', []) + ['Structuring mapping value @ key ' + repr(k)]"
             )
             lines.append("      errors.append(e)")
             lines.append("      continue")
@@ -698,12 +745,12 @@ def make_mapping_structure_fn(
             lines.append("      res[key] = value")
             lines.append("    except Exception as e:")
             lines.append(
-                "      e.__note__ = 'Structuring mapping key @ key ' + repr(k)"
+                "      e.__notes__ = getattr(e, '__notes__', []) + ['Structuring mapping key @ key ' + repr(k)]"
             )
             lines.append("      errors.append(e)")
             lines.append("  if errors:")
             lines.append(
-                f"    raise IterableValidationError('While structuring {cl!r}', errors, __cattr_mapping_cl)"
+                f"    raise IterableValidationError('While structuring ' + {repr(cl)!r}, errors, __cattr_mapping_cl)"
             )
         else:
             lines.append(f"  res = {{{k_s}: {v_s} for k, v in mapping.items()}}")
@@ -720,7 +767,7 @@ def make_mapping_structure_fn(
     return fn
 
 
-def _generate_unique_filename(cls, func_name, reserve=True):
+def _generate_unique_filename(cls: Any, func_name: str, reserve: bool = True) -> str:
     """
     Create a "filename" suitable for a function being generated.
     """
